/*********************************************************************
 * TipMe - full featured tip calculator for the Palm Pilot
 * Alan Harder - v1.22 - 4/27/98
 * Alan.Harder@Sun.COM     http://www.moshpit.org/pilot
 * Freeware - feel free to distribute and/or modify this code.. just
 *            leave my name on it.
 ********************************************************************/
#include "tipme.h"
#include <GuiTools.h>
#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#define CREATOR		'TpMe'
#define PREF_VERSION	3

Char deciSep[] = ".", deciNoDime[] = ".0";
Char deciChr = '.';

// App Preferences Type
//
typedef struct
{
    Int taxRate, tipRate, roundType, tipOn, roundAmt;
    Boolean useTax;
}
TipPrefsType;

// Methods to get and set Amounts in text fields.  An amount is the
// number of CENTS, and will be displayed with the decimal point placed
// appropriately..
//
void PutAmountInField(FieldPtr fieldPtr, Long amount)
{
    CharPtr str;
    Long amt;

    str = (CharPtr)MemPtrNew(16);
    if (amount < 0)
    {
	amt = -amount;
	StrCopy(str, "(");
    }
    else
    {
	amt = amount;
	StrCopy(str, "");
    }
    StrIToA(str + StrLen(str), amt/100);
    StrCat(str, (amt%100 < 10) ? deciNoDime : deciSep);
    StrIToA(str + StrLen(str), amt%100);
    if (amount < 0)
	StrCat(str, ")");

    SetText(fieldPtr, str);
    MemPtrFree(str);
}

Long GetAmountFromField(FieldPtr fieldPtr)
{
    Long val;
    Int cents;
    CharPtr str, str2;
    str = GetText(fieldPtr);
    if (str == NULL) return -1;
    if (StrLen(str) == 0) { MemPtrFree(str); return -1; }
    val = StrAToI(str) * 100;
    str2 = StrChr(str, deciChr);
    if (str2 != NULL && str2[1] >= '0' && str2[1] <= '9')
    {
	cents = (str2[1] - '0') * 10;
	if (str2[2] >= '0' && str2[2] <= '9')
	    cents += str2[2] - '0';
	val += cents;
    }
    MemPtrFree(str);
    return val;
}


DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    EventType evt;
    Word error, prefsSize;
    TipPrefsType *prefs;
    SystemPreferencesType syspref;

    Int people, diff, type;
    Long total, share, amount, tip, tax, tipbase, roundby;
    FieldPtr amountFld, taxFld, taxAmtFld, tipFld, tipAmtFld,
	     totalFld, peopleFld, shareFld, roundAmtFld;
    ControlPtr taxCB, rndUpBtn, rndDownBtn;
    ListPtr rndType, tipOn;

    if (cmd != sysAppLaunchCmdNormalLaunch)
	return 0;

    PrefGetPreferences(&syspref);
    type = syspref.numberFormat;
    if (type==nfPeriodComma || type==nfSpaceComma)
    {
	deciChr = deciSep[0] = deciNoDime[0] = ',';
    }

    FrmGotoForm(myForm);

    prefs = MemPtrNew(sizeof(TipPrefsType));
    if (!PrefGetAppPreferences(CREATOR, PREF_VERSION,
			       prefs, sizeof(TipPrefsType)))
    {
	prefs->tipRate = prefs->taxRate = prefs->roundType = 0;
	prefs->tipOn = 2;
	prefs->useTax = false;
	prefs->roundAmt = 100;
    }

    while (1)
    {
	EvtGetEvent(&evt, evtWaitForever);
	if (SysHandleEvent(&evt))
	    continue;
	if (MenuHandleEvent(NULL, &evt, &error))
	    continue;

	switch (evt.eType)
	{
	  case ctlSelectEvent:
	    switch (evt.data.ctlSelect.controlID)
	    {
	      case calcBtn:
		amount = GetAmountFromField(amountFld);
		if (amount >= 0)
		    PutAmountInField(amountFld, amount);
		tip = GetAmountFromField(tipFld);
		if (tip >= 0)
		    PutAmountInField(tipFld, tip);
		if (amount < 0 || tip < 0)
		{
		    SndPlaySystemSound(sndWarning);
		    SetFocus( (tip < 0) ? tipField : amountField );
		    break;
		}
		type = LstGetSelection(tipOn);
		tax = GetAmountFromField(taxFld);
		if (tax >= 0) PutAmountInField(taxFld, tax);
		else          tax = 0;
		if (CtlGetValue(taxCB) > 0)
		{
		    tax = (amount*tax+5000)/10000;  // standard rounding
		    PutAmountInField(taxAmtFld, tax);
		} else if (type == 1) {  // pre-tax calc
		    tax = -(amount*tax*10 / (10000+tax) + 5)/10;
		    if (tax > 0)	// in case of integer overflow
			tax = -(amount*tax / (10000+tax));
		    PutAmountInField(taxAmtFld, tax);
		} else {
		    tax = 0;
		    SetText(taxAmtFld, "");
		}
		if (type == 0 || (type == 1 && tax >= 0))
		{
		    tipbase = amount;
		}
		else
		{
		    tipbase = amount + tax;
		}
		tip = (tipbase*tip+9999)/10000;   // always round up
		PutAmountInField(tipAmtFld, tip);
		if (tax < 0) tax = 0;
		total = amount + tax + tip;
		PutAmountInField(totalFld, total);
		people = GetNumFromField(peopleFld);
		if (people > 0)
		{
		    PutNumInField(peopleFld, people);
		    share = (total + people - 1) / people; // always round up
		    PutAmountInField(shareFld, share);
		}
		else
		{
		    share = -1;
		    SetText(shareFld, "");
		    SetText(peopleFld, "");
		}
		// enable rounding buttons:
		CtlSetLabel(rndUpBtn, "\001");
		CtlSetEnabled(rndUpBtn, true);
		CtlSetLabel(rndDownBtn, "\002");
		CtlSetEnabled(rndDownBtn, true);
		FrmDrawForm(FrmGetActiveForm());
		break;

	      case clearBtn:
		for (error = 0; error < 6; error++)
		    SetTextById(amountField + error, "");
		if (prefs->tipRate > 0)
		    PutAmountInField(tipFld, prefs->tipRate);
		if (prefs->taxRate > 0)
		    PutAmountInField(taxFld, prefs->taxRate);
		CtlSetLabel(rndUpBtn, "\003");
		CtlSetEnabled(rndUpBtn, false);
		CtlSetLabel(rndDownBtn, "\004");
		CtlSetEnabled(rndDownBtn, false);
		SetFocus(amountField);
		FrmDrawForm(FrmGetActiveForm());
		break;

	      case roundUpBtn:
	      case roundDownBtn:
		roundby = GetAmountFromField(roundAmtFld);
		if (roundby <= 0) break;
		type = LstGetSelection(rndType);
		tip = GetAmountFromField(tipAmtFld);
		diff = (evt.data.ctlSelect.controlID==roundUpBtn) ? roundby :0;

		switch (type)
		{
		  case 0: // total
		    diff -= total%roundby;
		    if (diff == 0) diff = -roundby;
		    break;
		  case 1: // tip
		    diff -= tip%roundby;
		    if (diff == 0) diff = -roundby;
		    break;
		  case 2: // share
		    diff -= share%roundby;
		    if (diff == 0) diff = -roundby;
		    diff = (share + diff) * people - total;
		    break;
		}
		if ((total + diff) < 0 || (tip + diff) < 0)
		    break;
		total += diff;
		tip += diff;
		if (people > 0)
		{
		    share = (total + people - 1) / people;
		    PutAmountInField(shareFld, share);
		}
		PutAmountInField(totalFld, total);
		PutAmountInField(tipAmtFld, tip);

		tip = tip * 10000 / tipbase;
		PutAmountInField(tipFld, tip);
		FrmDrawForm(FrmGetActiveForm());
		break;

	      case resetTipBtn:
		if (prefs->tipRate > 0)
		{
		    PutAmountInField(tipFld, prefs->tipRate);
		    FldDrawField(tipFld);
		}
		break;

	      case resetAmtBtn:
		SetText(amountFld, "");
		SetFocus(amountField);
		FldDrawField(amountFld);
		break;

	      default:
		FrmHandleEvent(FrmGetActiveForm(), &evt);
	    }
	    break;

	  case frmLoadEvent:
	    FrmSetActiveForm(FrmInitForm(evt.data.frmLoad.formID));
	    if (evt.data.frmLoad.formID == myForm)
	    {
		FormPtr frm = FrmGetActiveForm();
		amountFld = (FieldPtr)GetObjectPtrFromForm(frm, amountField);
		taxFld = (FieldPtr)GetObjectPtrFromForm(frm, taxField);
		taxAmtFld = (FieldPtr)GetObjectPtrFromForm(frm,taxAmountField);
		tipFld = (FieldPtr)GetObjectPtrFromForm(frm, tipField);
		tipAmtFld = (FieldPtr)GetObjectPtrFromForm(frm,tipAmountField);
		totalFld = (FieldPtr)GetObjectPtrFromForm(frm, totalField);
		peopleFld = (FieldPtr)GetObjectPtrFromForm(frm,peopleField);
		shareFld = (FieldPtr)GetObjectPtrFromForm(frm, shareField);
		taxCB = (ControlPtr)GetObjectPtrFromForm(frm, taxCBox);
		rndType = (ListPtr)GetObjectPtrFromForm(frm, roundList);
		rndUpBtn = (ControlPtr)GetObjectPtrFromForm(frm, roundUpBtn);
		rndDownBtn = (ControlPtr)GetObjectPtrFromForm(frm,roundDownBtn);
		roundAmtFld = (FieldPtr)GetObjectPtrFromForm(frm,roundAmtField);
		tipOn = (ListPtr)GetObjectPtrFromForm(frm, tiponList);
		if (prefs->tipRate > 0)
		    PutAmountInField(tipFld, prefs->tipRate);
		if (prefs->taxRate > 0)
		    PutAmountInField(taxFld, prefs->taxRate);
		if (prefs->roundAmt > 0)
		    PutAmountInField(roundAmtFld, prefs->roundAmt);
		if (prefs->useTax)
		    CtlSetValue(taxCB, 1);
		if (prefs->roundType > 0)
		{
		    LstSetSelection(rndType, prefs->roundType);
		    CtlSetLabel(GetObjectPtrFromForm(frm, roundTrig),
				LstGetSelectionText(rndType,prefs->roundType));
		}
		if (prefs->tipOn > 0)
		{
		    LstSetSelection(tipOn, prefs->tipOn);
		    CtlSetLabel(GetObjectPtrFromForm(frm, tiponTrig),
				LstGetSelectionText(tipOn, prefs->tipOn));
		}
	    }
	    break;

	  case frmOpenEvent:
	    FrmDrawForm(FrmGetActiveForm());
	    if (evt.data.frmOpen.formID == myForm)
	    {
		SetFocus( (prefs->tipRate > 0) ? amountField : tipField );
	    }
	    break;

	  case menuEvent:
	    if (evt.data.menu.itemID == menuSave)
	    {
		Long intVal;
		intVal = GetAmountFromField(tipFld);
		if (intVal >= 0)
		    prefs->tipRate = intVal;
		intVal = GetAmountFromField(taxFld);
		if (intVal >= 0)
		    prefs->taxRate = intVal;
	    }
	    else
		FrmHandleEvent(FrmGetActiveForm(), &evt);
	    break;

	  case appStopEvent:
	    {
		Long intVal;
		prefs->roundType = LstGetSelection(rndType);
		prefs->tipOn = LstGetSelection(tipOn);
		prefs->useTax = (CtlGetValue(taxCB) != 0);
		intVal = GetAmountFromField(roundAmtFld);
		if (intVal >= 0)
		    prefs->roundAmt = intVal;
		PrefSetAppPreferences(CREATOR, PREF_VERSION,
				      prefs, sizeof(TipPrefsType));
		MemPtrFree(prefs);
	    }
	    FrmCloseAllForms();
	    return 0;

	  default:
	    FrmHandleEvent(FrmGetActiveForm(), &evt);
	}
    }
    return 0;
}

